using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using FarseerPhysics.Dynamics;
using FarseerPhysics.Factories;
using Farseer331_Setup.PhysicsObjects;
using Farseer331_Setup.Helpers;


namespace Farseer331_Setup
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Player player;

        World world;
        List<DrawablePhysicsObject> crateList;        
        DrawablePhysicsObject floor;
        List<DrawablePhysicsObject> paddles;

        KeyboardState prevKeyboardState;
        Random random;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);                 
            Content.RootDirectory = "Content";

            // Prefer multisampling to get rid of all the jagged
            // edges of the boxes that we're going to draw
            graphics.PreferMultiSampling = true;       
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {            
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            random = new Random();
            
            // World is the most important farseer object. It's where
            // all the objects should be registered and it handles the 
            // entire simulation via the step function.
            // Here we instantiate it with earth like gravity
            world = new World(new Vector2(0, 9.8f));       
            
            floor = new DrawablePhysicsObject(world, Content.Load<Texture2D>("Floor"), new Vector2(GraphicsDevice.Viewport.Width, 100.0f), 1000);
            floor.Position = new Vector2(GraphicsDevice.Viewport.Width / 2.0f, GraphicsDevice.Viewport.Height - 50);
            floor.Body.BodyType = BodyType.Static;            

            crateList = new List<DrawablePhysicsObject>();

            paddles = new List<DrawablePhysicsObject>();

            

            // Create a simple paddle which center is anchored
            // in the background. It can rotate freely
            DrawablePhysicsObject simplePaddle = new DrawablePhysicsObject
            (
                world,
                Content.Load<Texture2D>("Paddle"),
                new Vector2(128, 16),
                10
            );

            JointFactory.CreateFixedRevoluteJoint
            (
                world,
                simplePaddle.Body,
                CoordinateHelper.ToWorld(new Vector2(0, 0)),
                CoordinateHelper.ToWorld(new Vector2(GraphicsDevice.Viewport.Width / 2.0f - 150, GraphicsDevice.Viewport.Height - 300))
            );

            paddles.Add(simplePaddle);


            // Creates a motorized paddle which left side is anchored in the background
            // it will rotate slowly but the motor is not set soo strong that
            // it can push everything away.
            DrawablePhysicsObject motorPaddle = new DrawablePhysicsObject
            (
                world,
                Content.Load<Texture2D>("Paddle"),
                new Vector2(128, 16),
                10
            );

            var j = JointFactory.CreateFixedRevoluteJoint
            (
                world,
                motorPaddle.Body,
                CoordinateHelper.ToWorld(new Vector2(-48, 0)),
                CoordinateHelper.ToWorld(new Vector2(GraphicsDevice.Viewport.Width / 2.0f, GraphicsDevice.Viewport.Height - 280))
            );

            // rotate 1/4 of a circle per second
            j.MotorSpeed = MathHelper.PiOver2;

            // have little torque (power) so it can push away a few blocks
            j.MotorTorque = 3; 
            j.MotorEnabled = true;
            j.MaxMotorTorque = 10;

            paddles.Add(motorPaddle);

            // Use two line joints (a sort of springs) to create a trampoline
            DrawablePhysicsObject trampolinePaddle = new DrawablePhysicsObject
            (
                world,
                Content.Load<Texture2D>("Paddle"),
                new Vector2(128, 16),
                10
            );

            trampolinePaddle.Position = new Vector2(600, floor.Position.Y - 175);

            var l = JointFactory.CreateLineJoint
            (
                floor.Body,
                trampolinePaddle.Body,
                CoordinateHelper.ToWorld(trampolinePaddle.Position - new Vector2(64, 0)),
                Vector2.UnitY
            );

            l.CollideConnected = true;
            l.Frequency = 2.0f;
            l.DampingRatio = 0.05f;

            var r = JointFactory.CreateLineJoint
            (
                floor.Body,
                trampolinePaddle.Body,
                CoordinateHelper.ToWorld(trampolinePaddle.Position + new Vector2(64, 0)),
                Vector2.UnitY
            );

            r.CollideConnected = true;
            r.Frequency = 2.0f;
            r.DampingRatio = 0.05f;

            world.AddJoint(l);
            world.AddJoint(r);

            paddles.Add(trampolinePaddle);
           
            player = new Player
            (
                world,
                Content.Load<Texture2D>("Player"),
                Content.Load<Texture2D>("Wheel"),
                new Vector2(20, 75),
                100,
                new Vector2(430, 0)
            );            

            prevKeyboardState = Keyboard.GetState();

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Spawn a crate at a random position at the top of the screen
        /// </summary>
        private void SpawnCrate()
        {
            DrawablePhysicsObject crate;
            crate = new DrawablePhysicsObject(world, Content.Load<Texture2D>("Crate"), new Vector2(50.0f, 50.0f), 0.1f);
            crate.Position = new Vector2(random.Next(50, GraphicsDevice.Viewport.Width - 50), 1);

            crateList.Add(crate);
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();


            KeyboardState keyboardState = Keyboard.GetState();
            if (keyboardState.IsKeyDown(Keys.Space) && !prevKeyboardState.IsKeyDown(Keys.Space))
            {
                SpawnCrate();
            }

            if (keyboardState.IsKeyDown(Keys.LeftControl) && !prevKeyboardState.IsKeyDown(Keys.LeftControl))
            {
                player.Jump();
            }


            if (keyboardState.IsKeyDown(Keys.Left))
            {
                player.Move(Movement.Left);
            }
            else if (keyboardState.IsKeyDown(Keys.Right))
            {
                player.Move(Movement.Right);
            }
            else
            {
                player.Move(Movement.Stop);
            }

            prevKeyboardState = keyboardState;

            world.Step((float)gameTime.ElapsedGameTime.TotalSeconds);


            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend);

            foreach (DrawablePhysicsObject crate in crateList)
            {
                crate.Draw(spriteBatch);
            }

            foreach (DrawablePhysicsObject paddle in paddles)
            {
                paddle.Draw(spriteBatch);
            }

            player.Draw(spriteBatch);

            floor.Draw(spriteBatch);
            
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
